@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/attr.c,v 1.9 2006/02/10 23:21:18 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/attr.c,v 1.14 2009/09/30 14:53:55 william Exp $
  */
 
 #define _INCLUDE_FROM_ATTR_C_
@@ -309,6 +309,7 @@ struct AttrRec *AddAttrByNameAndValue(ObjPtr, AttrName, AttrValue)
    pStrSeg->style = curStyle;
    pStrSeg->sz_unit = GetCurSzUnit();
    pStrSeg->underline_on = curUnderlineOn;
+   pStrSeg->overline_on = curOverlineOn;
 
    pStrSeg->asc = canvasFontAsc;
    pStrSeg->des = canvasFontDes;
@@ -322,6 +323,8 @@ struct AttrRec *AddAttrByNameAndValue(ObjPtr, AttrName, AttrValue)
    text_obj_ptr->y = ObjPtr->obbox.rby;
    text_obj_ptr->type = OBJ_TEXT;
    text_obj_ptr->color = ObjPtr->color;
+   memcpy(text_obj_ptr->color_str, ObjPtr->color_str,
+         sizeof(text_obj_ptr->color_str));
    text_obj_ptr->dirty = FALSE;
    text_obj_ptr->id = objId++;
    text_obj_ptr->rotation = 0;
@@ -2564,14 +2567,14 @@ int InitAttr()
          NULL) {
       maxAttrGroups = atoi(c_ptr);
    }
-#ifdef _TANGRAM_II
+   if (cmdLineTgrm2) {
 #ifdef DEFATTRGROUP
-   if (maxAttrGroups == 0) {
-      maxAttrGroups = 1;
-      attr_group_in_xdef = FALSE;
-   }
+      if (maxAttrGroups == 0) {
+         maxAttrGroups = 1;
+         attr_group_in_xdef = FALSE;
+      }
 #endif /* DEFATTRGROUP */
-#endif /* _TANGRAM_II */
+   }
    if (maxAttrGroups > 0) {
       gAttrGroupInfo = (AttrGroupInfo**)malloc(maxAttrGroups *
             sizeof(AttrGroupInfo*));
@@ -2592,8 +2595,7 @@ int InitAttr()
                      TOOL_NAME, buf);
                fprintf(stderr, "\n");
             }
-         } else {
-#ifdef _TANGRAM_II
+         } else if (cmdLineTgrm2) {
 #ifdef DEFATTRGROUP
             char *tmp_buf=UtilStrDup(DEFATTRGROUP);
 
@@ -2603,7 +2605,6 @@ int InitAttr()
             }
             UtilFree(tmp_buf);
 #endif /* DEFATTRGROUP */
-#endif /* _TANGRAM_II */
          }
       }
    } else if (maxAttrGroups == 0) {
