@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/auxtext.c,v 1.4 2006/02/10 23:21:18 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/auxtext.c,v 1.9 2009/09/30 14:53:55 william Exp $
  */
 
 #define _INCLUDE_FROM_AUXTEXT_C_
@@ -1448,7 +1448,7 @@ void DumpTextFill(FP, ObjPtr)
          if (preDumpSetup) PSUseColorPattern();
          DumpRectPath(FP, ltx, lty, rbx, rby, 3, FALSE);
          fprintf(FP, "%s %s\n", gPsCmd[PS_EOCLIP], gPsCmd[PS_NEWPATH]);
-         DumpPatFill(FP, fill, 8, ObjPtr->bbox, 3, TRUE);
+         DumpPatFill(FP, fill, ObjPtr->bbox, 3, TRUE);
       } else {
          DumpRectPath(FP, ltx, lty, rbx, rby, 3, FALSE);
          fprintf(FP, "%s\n", gPsCmd[PS_FILL]);
@@ -1978,6 +1978,7 @@ StrSegInfo *CreateStrSegFromString(buf, pOwnerStrBlock)
    pStrSeg->direction = canvasFontDirection;
    pStrSeg->dontreencode = canvasFontDontReencode;
    pStrSeg->underline_on = curUnderlineOn;
+   pStrSeg->overline_on = curOverlineOn;
 
    pStrSeg->asc = canvasFontAsc;
    pStrSeg->des = canvasFontDes;
@@ -2101,6 +2102,10 @@ struct ObjRec *FormTextObjectFromFile(FP, AbsX, AbsBaselineY)
    obj_ptr->y = AbsBaselineY-text_ptr->minilines.first->asc;
    obj_ptr->type = OBJ_TEXT;
    obj_ptr->color = colorIndex;
+   if (mainDisplay != NULL) {
+      UtilStrCpyN(obj_ptr->color_str, sizeof(obj_ptr->color_str),
+            colorMenuItems[colorIndex]);
+   }
    obj_ptr->id = objId++;;
    obj_ptr->dirty = FALSE;
    obj_ptr->rotation = 0;
