@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/choose.c,v 1.7 2006/02/10 23:21:19 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/choose.c,v 1.10 2009/08/13 01:14:20 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_CHOOSE_C_
@@ -446,7 +446,7 @@ void DrawCheckbox(dpy, win, gc, x, y, w, h, checked)
          XDrawLine(dpy, win, gc, x, y+h, x+w, y);
       }
    }
-   if (check_bits) { }
+   if (check_bits == NULL) { }
 }
 
 static
