@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/color.c,v 1.7 2006/02/10 23:21:19 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/color.c,v 1.16 2009/08/13 01:14:20 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_COLOR_C_
@@ -62,6 +62,7 @@
 #include "stk.e"
 #include "strtbl.e"
 #include "text.e"
+#include "tgcwheel.e"
 #include "util.e"
 #include "xpixmap.e"
 
@@ -210,7 +211,6 @@ int TgifParseColor(psz_color, p_color)
    return (int)XParseColor(mainDisplay, mainColormap, psz_color, p_color);
 }
 
-static
 int ParseAndAllocColorByName(colorname, color, red_req, green_req, blue_req)
    char *colorname;
    XColor *color;
@@ -1242,6 +1242,10 @@ int ChangeObjColor(ObjPtr, ColorIndex)
          }
       } else if (ObjPtr->color != ColorIndex) {
          ObjPtr->color = ColorIndex;
+         if (mainDisplay != NULL) {
+            UtilStrCpyN(ObjPtr->color_str, sizeof(ObjPtr->color_str),
+                  colorMenuItems[ObjPtr->color]);
+         }
          if (ObjPtr->type == OBJ_XPM) {
             /* RecolorXPmObj(ObjPtr, ColorIndex); */
          }
@@ -1556,7 +1560,7 @@ void CleanUpColors()
    defaultColorIndex = 9;
    colorIndex = 0;
 
-   if (upend_bits) { }
+   if (upend_bits == NULL) { }
 }
 
 static
@@ -1798,7 +1802,6 @@ void ColorDummyEventHandler(input)
 
 typedef struct tagUpdatePixelRec {
    int picture_changed;
-   char **old_color_str;
 } UpdatePixelInfo;
 
 static
@@ -1808,25 +1811,27 @@ void UpdatePixelForStrSeg(pStrSeg, pUserData)
 {
    UpdatePixelInfo *pupi=((UpdatePixelInfo*)pUserData);
    int new_alloc=FALSE;
-   int index=QuickFindColorIndex(NULL, pupi->old_color_str[pStrSeg->color],
-         &new_alloc, TRUE);
+   int index=QuickFindColorIndex(NULL, pStrSeg->color_str, &new_alloc, TRUE);
 
    if (index != pStrSeg->color) pupi->picture_changed = TRUE;
    if (index == INVALID) {
       sprintf(gszMsgBox,
             TgLoadCachedString(CSTID_CANNOT_ALLOC_COLOR_USE_ALT),
-            pupi->old_color_str[pStrSeg->color], colorMenuItems[colorIndex]);
+            pStrSeg->color_str, colorMenuItems[colorIndex]);
       Msg(gszMsgBox);
       allocColorFailed = TRUE;
       pStrSeg->color = colorIndex;
    } else {
       pStrSeg->color = index;
    }
+   if (mainDisplay != NULL) {
+      UtilStrCpyN(pStrSeg->color_str, sizeof(pStrSeg->color_str),
+            colorMenuItems[pStrSeg->color]);
+   }
 }
 
-int UpdatePixel(ObjPtr, OldColorStr)
+int UpdatePixel(ObjPtr)
    struct ObjRec *ObjPtr;
-   char **OldColorStr;
 {
    int c, i, r;
    int ncolors=0, new_alloc=FALSE, index=0, changed=FALSE, len=0;
@@ -1843,25 +1848,29 @@ int UpdatePixel(ObjPtr, OldColorStr)
    case OBJ_PIN:
       for (obj_ptr=ObjPtr->detail.r->last; obj_ptr!=NULL;
             obj_ptr=obj_ptr->prev) {
-         if (UpdatePixel(obj_ptr, OldColorStr)) {
+         if (UpdatePixel(obj_ptr)) {
             picture_changed = TRUE;
          }
       }
       break;
    case OBJ_XPM:
-      index = QuickFindColorIndex(NULL, OldColorStr[ObjPtr->color],
+      index = QuickFindColorIndex(NULL, ObjPtr->color_str,
             &new_alloc, TRUE);
       if (index != ObjPtr->color) picture_changed = TRUE;
       if (index == INVALID) {
          sprintf(gszMsgBox,
                TgLoadCachedString(CSTID_CANNOT_ALLOC_COLOR_USE_ALT),
-               OldColorStr[ObjPtr->color], colorMenuItems[colorIndex]);
+               ObjPtr->color_str, colorMenuItems[colorIndex]);
          Msg(gszMsgBox);
          allocColorFailed = TRUE;
          ObjPtr->color = colorIndex;
       } else {
          ObjPtr->color = index;
       }
+      if (mainDisplay != NULL) {
+         UtilStrCpyN(ObjPtr->color_str, sizeof(ObjPtr->color_str),
+               colorMenuItems[ObjPtr->color]);
+      }
       changed = FALSE;
       xpm_ptr = ObjPtr->detail.xpm;
       ncolors = xpm_ptr->ncolors;
@@ -1936,25 +1945,28 @@ int UpdatePixel(ObjPtr, OldColorStr)
       free(to_pixels);
       break;
    default:
-      index = QuickFindColorIndex(NULL, OldColorStr[ObjPtr->color],
+      index = QuickFindColorIndex(NULL, ObjPtr->color_str,
             &new_alloc, TRUE);
       if (index != ObjPtr->color) picture_changed = TRUE;
       if (index == INVALID) {
          sprintf(gszMsgBox,
                TgLoadCachedString(CSTID_CANNOT_ALLOC_COLOR_USE_ALT),
-               OldColorStr[ObjPtr->color], colorMenuItems[colorIndex]);
+               ObjPtr->color_str, colorMenuItems[colorIndex]);
          Msg(gszMsgBox);
          allocColorFailed = TRUE;
          ObjPtr->color = colorIndex;
       } else {
          ObjPtr->color = index;
       }
+      if (mainDisplay != NULL) {
+         UtilStrCpyN(ObjPtr->color_str, sizeof(ObjPtr->color_str),
+               colorMenuItems[ObjPtr->color]);
+      }
       if (ObjPtr->type == OBJ_TEXT) {
          UpdatePixelInfo upi;
 
          memset(&upi, 0, sizeof(UpdatePixelInfo));
          upi.picture_changed = FALSE;
-         upi.old_color_str = OldColorStr;
          DoFuncOnStrSegForMiniLines(&ObjPtr->detail.t->minilines,
                UpdatePixelForStrSeg, &upi);
          if (upi.picture_changed) {
@@ -1964,7 +1976,7 @@ int UpdatePixel(ObjPtr, OldColorStr)
       break;
    }
    for (attr_ptr=ObjPtr->fattr; attr_ptr != NULL; attr_ptr=attr_ptr->next) {
-      if (UpdatePixel(attr_ptr->obj, OldColorStr)) {
+      if (UpdatePixel(attr_ptr->obj)) {
          picture_changed = TRUE;
       }
    }
@@ -1999,10 +2011,8 @@ void UpdateXPmObjects(ObjPtr)
 
 int FlushColormap()
 {
-   register int i;
-   int old_allocated_colors, changed=FALSE;
-   char **old_color_str;
-   struct ObjRec *obj_ptr;
+   int changed=FALSE;
+   struct ObjRec *obj_ptr=NULL;
 
 #ifdef DONT_FREE_COLORMAP
    Msg(TgLoadString(STID_COLORMAP_NOT_RESETED));
@@ -2025,15 +2035,6 @@ int FlushColormap()
             UpdateXPmObjects(obj_ptr);
          }
       }
-      old_color_str = (char**)malloc(maxColors*sizeof(char*));
-      if (old_color_str == NULL) FailAllocMessage();
-      old_allocated_colors = maxColors;
-      for (i=0; i < maxColors; i++) {
-         old_color_str[i] =
-               (char*)malloc((strlen(colorMenuItems[i])+1)*sizeof(char));
-         if (old_color_str[i] == NULL) FailAllocMessage();
-         strcpy(old_color_str[i], colorMenuItems[i]);
-      }
       initColorDontReload = TRUE;
       CleanUpColors();
       XFreeColormap(mainDisplay, mainColormap);
@@ -2086,7 +2087,7 @@ int FlushColormap()
       gnUpdatePixelObjCount = 0;
       for (page_ptr=firstPage; page_ptr != NULL; page_ptr=page_ptr->next) {
          for (obj_ptr=page_ptr->bot; obj_ptr!=NULL; obj_ptr=obj_ptr->prev) {
-            if (UpdatePixel(obj_ptr, old_color_str)) {
+            if (UpdatePixel(obj_ptr)) {
                changed = TRUE;
             }
          }
@@ -2094,14 +2095,12 @@ int FlushColormap()
       if (tgifObj->fattr != NULL) {
          for (attr_ptr=tgifObj->fattr; attr_ptr != NULL;
                attr_ptr=attr_ptr->next) {
-            if (UpdatePixel(attr_ptr->obj, old_color_str)) {
+            if (UpdatePixel(attr_ptr->obj)) {
                changed = TRUE;
             }
          }
       }
       RestoreStatusStrings();
-      for (i=0; i < old_allocated_colors; i++) free(old_color_str[i]);
-      free(old_color_str);
 
       DestroyPinnedMenu(MENU_COLOR);
       if (colorLayers) {
@@ -2143,6 +2142,11 @@ void AddColor()
    }
 }
 
+void ChooseColor()
+{
+   TgColorWheel_HandleCntrlSpace(mainDisplay, drawWindow);
+}
+
 void SetAltEditTextBgColor()
 {
    char spec[MAXSTRING+1];
