@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/const.h,v 1.4 2006/02/10 23:21:19 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/const.h,v 1.20 2009/07/28 14:33:55 william Exp $
  */
 
 #ifndef _TGIF_CONST_H_
@@ -185,12 +185,17 @@
 
 #define MARK(W,GC,X,Y) XFillRectangle(mainDisplay,W,GC, \
       (X)-handleSize,(Y)-handleSize,(handleSize<<1)+1,(handleSize<<1)+1)
+#define MARKHR(W,GC,X,Y) XDrawRectangle(mainDisplay,W,GC, \
+      (X)-handleSize,(Y)-handleSize,(handleSize<<1),(handleSize<<1))
 #define MARKV(W,GC,X,Y) XDrawLine(mainDisplay,W,GC, \
       (X)-(handleSize+1),(Y),(X)+(handleSize+1),(Y)); \
       XDrawLine(mainDisplay,W,GC,(X),(Y)-(handleSize+1),(X),(Y)+(handleSize+1))
 #define MARKO(W,GC,X,Y) XFillArc(mainDisplay,W,GC, \
       (X)-handleSize,(Y)-handleSize,(handleSize<<1)+1,(handleSize<<1)+1, \
       0,(360<<6))
+#define MARKHO(W,GC,X,Y) XDrawArc(mainDisplay,W,GC, \
+      (X)-handleSize,(Y)-handleSize,(handleSize<<1),(handleSize<<1), \
+      0,(360<<6))
 #define PtInMark(PtX,PtY,MarkX,MarkY) ((PtX) >= (MarkX)-handleSize && \
       (PtY) >= (MarkY)-handleSize && (PtX) <= (MarkX)+handleSize && \
       (PtY) <= (MarkY)+handleSize)
@@ -286,8 +291,9 @@
 #define LT_STRAIGHT 0
 #define LT_SPLINE 1
 #define LT_INTSPLINE 2
+#define LT_STRUCT_SPLINE 3
 
-#define MAXLINETYPES 3
+#define MAXLINETYPES 4
 
 #define LS_PLAIN 0
 #define LS_RIGHT 1
@@ -467,8 +473,9 @@
 #define JPEG_FILE 11
 #define PPM_FILE 12
 #define NETLIST_FILE 13
+#define SVG_FILE 14
 
-#define MAXDEFWHERETOPRINT 14
+#define MAXDEFWHERETOPRINT 15
 
 /* measurement */
 
@@ -709,8 +716,13 @@
 
 #define XPM_XPM  0
 #define XPM_JPEG 1
-#define XPM_EPS  2 /* not implemented */
-#define XPM_EPSI 3 /* not implemented */
+#define PPM_TRUE 2
+#define XPM_EPS  3 /* not implemented */
+#define XPM_EPSI 4 /* not implemented */
+
+#define PPM_DATA_RAW      0 /* not used */
+#define PPM_JPEG_COMPRESS 1
+#define PPM_DATA_DEFLATED 2
 
 /* cmd stuff */
 
@@ -754,6 +766,7 @@
 /* copypaste stuff */
 
 #define TGIF_HEADER 0x80
+#define TGIF_PROTOCOL_ATOM "_TGIF_PROTOCOL_ATOM"
 
 /* interrupt stuff */
 
@@ -985,4 +998,16 @@
 #define PROP_MASK_UNDERLINE_ON	0x00020000
 #define PROP_MASK_WIDTH_INDEX	0x00100000 /* GetProperty() only */
 
+/* mark stuff */
+
+#define FORWARD 0
+#define REVERSE 1
+
+/* auto rotate pivot stuff */
+
+#define AUTO_ROTATE_PIVOT 0
+#define USER_ROTATE_PIVOT 1
+
+#define MAX_ROTATE_PIVOT 2
+
 #endif /*_TGIF_CONST_H_*/
