@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/grid.c,v 1.3 2006/02/10 23:21:19 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/grid.c,v 1.7 2009/06/05 21:52:40 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_GRID_C_
@@ -568,6 +568,7 @@ void WhereToPrintMsg()
       case NETLIST_FILE:
          Msg(TgLoadString(STID_WILL_EXPORT_NETLIST_FILE));
          break;
+      case SVG_FILE: Msg(TgLoadString(STID_WILL_EXPORT_SVG_FILE)); break;
       }
    }
 }
@@ -1389,7 +1390,10 @@ int RefreshShowHideMenu(menu)
 
    if (gstWBInfo.do_whiteboard) {
       /* ShowChat */
+      ok &= TgEnableMenuItemById(menu, CMDID_TOGGLESHOWCHAT, TRUE);
       ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLESHOWCHAT, !noChatWindow);
+   } else {
+      ok &= TgEnableMenuItemById(menu, CMDID_TOGGLESHOWCHAT, FALSE);
    }
    return ok;
 }
