@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/inmethod.c,v 1.3 2006/02/10 23:21:20 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/inmethod.c,v 1.7 2009/10/02 15:31:24 william Exp $
  */
 
 #define _INCLUDE_FROM_INMETHOD_C_
@@ -59,6 +59,22 @@
 #include "strtbl.e"
 #include "tidget.e"
 
+#ifndef _NO_DL_SUPPORT
+#define CLOSE_DL(handle) dlclose(handle)
+#define OPEN_DL(path,flag) dlopen((path),(flag))
+#define GET_DL_SYM(handle,symbol) dlsym((handle),(symbol))
+#ifndef RTLD_NOW
+#define OPEN_DL_MODE 1
+#else /* RTLD_NOW */
+#define OPEN_DL_MODE (RTLD_NOW|RTLD_GLOBAL)
+#endif /* ~RTLD_NOW */
+#else /* _NO_DL_SUPPORT */
+#define CLOSE_DL(handle)
+#define OPEN_DL(path,flag) NULL
+#define GET_DL_SYM(handle,symbol) NULL
+#define OPEN_DL_MODE 0
+#endif /* ~_NO_DL_SUPPORT */
+
 int gnInputMethod=TGIM_NONE; /* one of TGIM_* */
 int gnOverTheSpot=FALSE;
 
@@ -650,8 +666,60 @@ static struct tagIMInfo gstIMInfo[] = {
    }
 };
 
+static void *gpLibIdnHandle=NULL;
+static Tgim_stringprep_convert_Func *gpfn_stringprep_convert=NULL;
+
+static
+void CleanUpTgim_stringprep_convert()
+{
+   if (gpLibIdnHandle != NULL) {
+      gpfn_stringprep_convert = NULL;
+      CLOSE_DL(gpLibIdnHandle);
+      gpLibIdnHandle = NULL;
+   }
+}
+
+static
+void InitTgim_stringprep_convert()
+{
+#if (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ))
+   gpLibIdnHandle = OPEN_DL("libidn.so", OPEN_DL_MODE);
+   if (gpLibIdnHandle != NULL) {
+      gpfn_stringprep_convert =
+            (Tgim_stringprep_convert_Func*)GET_DL_SYM(gpLibIdnHandle,
+            "stringprep_convert");
+      if (gpfn_stringprep_convert == NULL) {
+         CLOSE_DL(gpLibIdnHandle);
+         gpLibIdnHandle = NULL;
+      }
+   }
+#endif /* (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ)) */
+}
+
+int Tgim_has_stringprep_convert()
+{
+#if (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ))
+   if (gpLibIdnHandle != NULL && gpfn_stringprep_convert != NULL) {
+      return TRUE;
+   }
+#endif /* (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ)) */
+   return FALSE;
+}
+
+char *Tgim_stringprep_convert(const char *str, const char *to_codeset,
+      const char *from_codeset)
+{
+#if (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ))
+   if (gpLibIdnHandle != NULL && gpfn_stringprep_convert != NULL) {
+       return (gpfn_stringprep_convert)(str, to_codeset, from_codeset);
+   }
+#endif /* (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ)) */
+   return NULL;
+}
+
 void CleanUpInputMethods()
 {
+   CleanUpTgim_stringprep_convert();
    if (gnInputMethodIndex != INVALID &&
          gstIMInfo[gnInputMethodIndex].pCleanUpFunc != NULL) {
       (gstIMInfo[gnInputMethodIndex].pCleanUpFunc)(mainDisplay, mainWindow);
@@ -723,6 +791,10 @@ int InitInputMethods()
          fprintf(stderr, "\n");
       }
    }
+   if (gnSingleOrDoubleByteInputMethod == TGIM_SBIM &&
+         gnInputMethod != INVALID) {
+      InitTgim_stringprep_convert();
+   }
    return TRUE;
 }
 
