@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/mainmenu.c,v 1.4 2006/02/10 23:21:20 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/mainmenu.c,v 1.7 2009/01/09 22:25:30 william Exp $
  */
 
 #define _INCLUDE_FROM_MAINMENU_C_
@@ -984,9 +984,11 @@ void UpdatePinnedMenu(menu_index)
             case MENU_PAGE: RefreshPageMenu(menu); break;
             case MENU_PAGELAYOUT: RefreshPageLayoutMenu(menu); break;
             case MENU_HELP: RefreshHelpMenu(menu); break;
-#ifdef _TANGRAM_II
-            case MENU_TANGRAM2: RefreshTangram2Menu(menu); break;
-#endif /* _TANGRAM_II */
+            case MENU_TANGRAM2:
+               if (cmdLineTgrm2) {
+                  RefreshTangram2Menu(menu);
+               }
+               break;
 
             case MENU_MODE: RefreshModeMenu(menu); break;
 
