@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/msg.c,v 1.18 2006/06/12 22:04:33 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/msg.c,v 1.44 2009/10/02 20:14:03 william Exp $
  */
 
 #define _INCLUDE_FROM_MSG_C_
@@ -57,6 +57,7 @@
 #include "raster.e"
 #include "rect.e"
 #include "remote.e"
+#include "rm_intrf.e"
 #include "setup.e"
 #include "strtbl.e"
 #include "tangram2.e"
@@ -77,6 +78,8 @@ char	fatalMsgBuf1[MAXSTRING<<1];
 char	fatalMsgBuf2[MAXSTRING<<1];
 
 int	cmdLineParentWindowID=0;
+int	cmdLineTgrm2=FALSE;
+int	cmdLineWb2=FALSE;
 
 struct MsgRec {
    char			* s;
@@ -975,6 +978,37 @@ void RestoreStatusStringsFromBuf(ppsz_buf, one_line)
    }
 }
 
+void BeginProgress(ppi, total_size)
+   ProgressInfo *ppi;
+   int total_size;
+{
+   memset(ppi, 0, sizeof(ProgressInfo));
+   ppi->total_size = total_size;
+   SaveStatusStringsIntoBuf(ppi->status_buf, &ppi->one_line_status);
+}
+
+void UpdateProgress(ppi, cur_size)
+   ProgressInfo *ppi;
+   int cur_size;
+{
+   int percent=(int)(((double)cur_size) / ((double)ppi->total_size) *
+            ((double)100));
+
+   if (percent >= ppi->target_percent) {
+      sprintf(gszMsgBox, TgLoadCachedString(CSTID_PROGRESS_PERCENT),
+            percent);
+      SetStringStatus(gszMsgBox);
+      XSync(mainDisplay, False);
+      while (ppi->target_percent <= percent) ppi->target_percent += 5;
+   }
+}
+
+void EndProgress(ppi)
+   ProgressInfo *ppi;
+{
+   RestoreStatusStringsFromBuf(ppi->status_buf, ppi->one_line_status);
+}
+
 void MsgEventHandler(input)
    XEvent *input;
 {
@@ -1172,8 +1206,8 @@ void CleanUpStatus()
       free(topStatusInfo);
       topStatusInfo = next_mosi;
    }
-   if (redraw_bits) { }
-   if (btn1_bits) { }
+   if (redraw_bits == NULL) { }
+   if (btn1_bits == NULL) { }
 }
 
 static char	*scanVal=NULL;
@@ -1470,30 +1504,9 @@ void GetTgifVersionAndPatchLevel(buf, buf_sz)
             TGIF_DISTR_VER);
    }
 #endif /* TGIF_DISTR_VER */
-#if defined(_TANGRAM_II) || defined(_TGIF_WB2)
-   {
-#ifdef _TANGRAM_II
-      int tangram2=TRUE;
-#else /* ~_TANGRAM_II */
-      int tangram2=FALSE;
-#endif /* _TANGRAM_II */
-#ifdef _TGIF_WB2
-      int tgwb2=TRUE;
-#else /* ~_TGIF_WB2 */
-      int tgwb2=FALSE;
-#endif /* _TGIF_WB2 */
-      if (tangram2 && tgwb2) {
-         sprintf(&buf1[strlen(buf1)], TgLoadString(STID_TGIF_FLAGS),
-               "_TANGRAM_II, _TGIF_WB2");
-      } else if (tangram2) {
-         sprintf(&buf1[strlen(buf1)], TgLoadString(STID_TGIF_FLAGS),
-               "_TANGRAM_II");
-      } else if (tgwb2) {
-         sprintf(&buf1[strlen(buf1)], TgLoadString(STID_TGIF_FLAGS),
-               "_TGIF_WB2");
-      }
-   }
-#endif /* defined(_TANGRAM_II) || defined(_TGIF_WB2) */
+#ifdef NOT_DEFINED
+   sprintf(&buf1[strlen(buf1)], TgLoadString(STID_TGIF_FLAGS), "_TGIF_WB2");
+#endif /* NOT_DEFINED */
    UtilStrCpyN(buf, buf_sz, buf1);
 }
 
@@ -1513,6 +1526,7 @@ void Usage(tool_name)
    fprintf(stdout, "\t[-bd <color>] \\\n");
    fprintf(stdout, "\t[-rv] [-nv] \\\n");
    fprintf(stdout, "\t[-bw] \\\n");
+   fprintf(stdout, "\t[-tgwb2 [-rmcastlibdir <directory> | -rmcastlibpath <path>] [-parent <parent_window_id>]] \\\n");
    fprintf(stdout, "\t[-reqcolor] \\\n");
    fprintf(stdout, "\t[-cwo[+sbwarp]] \\\n");
    fprintf(stdout, "\t[-hyper] \\\n");
@@ -1527,12 +1541,7 @@ void Usage(tool_name)
    fprintf(stdout, "\t[-pdfspd | -pdfspd=true | -pdfspd=false] \\\n");
    fprintf(stdout, "\t[-pssetup \"<string>\"] \\\n");
    fprintf(stdout, "\t[-nomode] \\\n");
-#if defined(_TANGRAM_II) || defined(_TGIF_WB2)
-   fprintf(stdout, "\t[-parent <parent_window_id>] \\\n");
-#endif /* defined(_TANGRAM_II) || defined(_TGIF_WB2) */
-#ifdef _TANGRAM_II
    fprintf(stdout, "\t[-tg2genparse=\"<string>\"] \\\n");
-#endif /* _TANGRAM_II */
    fprintf(stdout, "\t[{<file>[.%s] | -merge <file1>[.%s] <file2>[.%s] ...}]\n",
          OBJ_FILE_EXT, OBJ_FILE_EXT, OBJ_FILE_EXT);
    fprintf(stdout, "or:\n");
@@ -1589,6 +1598,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
    *cmdLineGenParser = '\0';
    *cmdLineFileToExec = '\0';
    *cmdLineJpegToPpm6Cmd = '\0';
+   *cmdLineCustomPatternDir = '\0';
    cmdLineOneFilePerPage = FALSE;
    *outputDir = '\0';
    for (argc--, argv++; argc > 0; argc--, argv++) {
@@ -1596,13 +1606,15 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
          if (strcmp(*argv, "-version") == 0 ||
                strcmp(*argv, "--version") == 0) {
             show_copyright = TRUE;
+         } else if (strcmp(*argv, "-tgrm2") == 0) {
+            cmdLineTgrm2 = TRUE;
          } else if (strcmp(*argv, "-justversion") == 0) {
             char buf[MAXSTRING];
 
             GetTgifVersionAndPatchLevel(buf, sizeof(buf));
-            fprintf(stderr, "%s\n", buf);
-            fprintf(stderr, "%s (%s)\n\n", copyrightString, authorEmailString);
-            fflush(stderr);
+            fprintf(stdout, "%s\n", buf);
+            fprintf(stdout, "%s (%s)\n\n", copyrightString, authorEmailString);
+            fflush(stdout);
 
             return INVALID;
          }
@@ -1668,12 +1680,13 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
    c_ptr += strlen(printCommand);
    for (argc--, argv++; argc > 0; argc--, argv++) {
       if (**argv == '-') {
-         int len;
+         int len=0;
 
          if (strcmp(*argv, "-display") == 0) {
             argc--; argv++;
             if (argc <= 0) {
-               fprintf(stderr, "%s\n", TgLoadString(STID_MISSING_DISPLAY_NAME));
+               fprintf(stderr, "%s\n\n",
+                     TgLoadString(STID_MISSING_DISPLAY_NAME));
                Usage(progName);
             }
             if ((displayName=UtilStrDup(*argv)) == NULL) FailAllocMessage();
@@ -1730,6 +1743,8 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             whereToPrint = cmdLineWhereToPrint = TEXT_FILE;
          } else if (strcmp(*argv, "-netlist") == 0) {
             whereToPrint = cmdLineWhereToPrint = NETLIST_FILE;
+         } else if (strcmp(*argv,"-svg") == 0) {
+            whereToPrint = cmdLineWhereToPrint = SVG_FILE;
          } else if (strcmp(*argv, "-raw") == 0) {
             cmdLineDumpURL = TRUE;
             cmdLineDumpURLWithHeader = FALSE;
@@ -1751,7 +1766,8 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
          } else if (strcmp(*argv, "-page") == 0) {
             argc--; argv++;
             if (argc <= 0) {
-               fprintf(stderr, "%s\n", TgLoadString(STID_MISSING_PAGE_NUMBER));
+               fprintf(stderr, "%s\n\n",
+                     TgLoadString(STID_MISSING_PAGE_NUMBER));
                Usage(progName);
             }
             cmdLineHasPageNum = TRUE;
@@ -1761,7 +1777,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
                cmdLineHasPageNum = FALSE;
                fprintf(stderr, TgLoadString(STID_INVALID_GIVEN_PAGE_NUMBER_STR),
                      cmdLinePageNumStr);
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
          } else if (strcmp(*argv, "-one_file_per_page") == 0) {
@@ -1794,7 +1810,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (argc <= 0) {
                fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                      "-pssetup");
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
             cmdLinePsSetup = TRUE;
@@ -1814,7 +1830,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (argc <= 0) {
                fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                      "-bop_hook");
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
             SetBopHook(*argv);
@@ -1823,7 +1839,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (argc <= 0) {
                fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                      "-eop_hook");
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
             SetEopHook(*argv);
@@ -1834,7 +1850,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (argc <= 0) {
                fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                      "-tmp_file_mode");
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
             UtilStrCpyN(cmdLineTmpFileModeStr, sizeof(cmdLineTmpFileModeStr),
@@ -1848,7 +1864,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
                   fprintf(stderr,
                         TgLoadString(STID_INVALID_ARG_FOR_CMDLINE_OPT),
                         "-tmp_file_mode", cmdLineTmpFileModeStr);
-                  fprintf(stderr, "\n");
+                  fprintf(stderr, "\n\n");
                   Usage(progName);
                }
             }
@@ -1897,7 +1913,6 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (*cmdLineProducedBy != '\0') {
                UtilRemoveQuotes(cmdLineProducedBy);
             }
-#ifdef _TANGRAM_II
          } else if (strncmp(*argv, "-tg2genparser=", 14) == 0) {
             UtilStrCpyN(cmdLineGenParser, sizeof(cmdLineGenParser),
                   &(*argv)[14]);
@@ -1905,7 +1920,6 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (*cmdLineGenParser != '\0') {
                UtilRemoveQuotes(cmdLineGenParser);
             }
-#endif /* _TANGRAM_II */
          } else if (strcmp(*argv, "-parse_html") == 0) {
             cmdLineParseHtml = TRUE;
             /* cmdLineDumpURL will be reset in UrlDump() */
@@ -1917,13 +1931,14 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (argc <= 0) {
                fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                      "-exec");
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
             UtilStrCpyN(cmdLineFileToExec, sizeof(cmdLineFileToExec), *argv);
             UtilTrimBlanks(cmdLineFileToExec);
             if (*cmdLineFileToExec == '-' && cmdLineFileToExec[1] != '\0') {
-               fprintf(stderr, "%s\n", TgLoadString(STID_BAD_EXEC_FNAME_DASH));
+               fprintf(stderr, "%s\n\n",
+                     TgLoadString(STID_BAD_EXEC_FNAME_DASH));
                Usage(progName);
             }
             cmdLineOpenDisplay = TRUE;
@@ -1932,7 +1947,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (argc <= 0) {
                fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                      "-tgif_path");
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
             len = strlen(*argv);
@@ -1951,7 +1966,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (argc <= 0) {
                fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                      "-print_cmd");
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
             len = strlen(*argv);
@@ -1966,7 +1981,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (argc <= 0) {
                fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                      "-j2p6_cmd");
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
             len = strlen(*argv);
@@ -1989,6 +2004,28 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
                   cmdLineJpegToPpm6Cmd[strlen(cmdLineJpegToPpm6Cmd)-1] = '\0';
                }
             }
+         } else if (strcmp(*argv, "-patterndir") == 0) {
+            argc--; argv++;
+            if (argc <= 0) {
+               fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
+                     "-patterndir");
+               fprintf(stderr, "\n\n");
+               Usage(progName);
+            }
+            UtilStrCpyN(cmdLineCustomPatternDir,
+                  sizeof(cmdLineCustomPatternDir), *argv);
+            UtilTrimBlanks(cmdLineCustomPatternDir);
+            len = strlen(cmdLineCustomPatternDir);
+            if (len > 0 && cmdLineCustomPatternDir[len-1] == DIR_SEP) {
+               cmdLineCustomPatternDir[len-1] = '\0';
+            }
+            if (*cmdLineCustomPatternDir == '\0' ||
+                  !UtilIsDirectory(cmdLineCustomPatternDir)) {
+               fprintf(stderr, TgLoadString(STID_PATTERNDIR_NOT_EXIST),
+                     cmdLineCustomPatternDir);
+               fprintf(stderr, "\n");
+               *cmdLineCustomPatternDir = '\0';
+            }
          } else if (strncmp(*argv, "-o", 2) == 0) {
             strcpy(outputDir, &(*argv)[2]);
          } else if (strcmp(*argv, "-prtgif") != 0 &&
@@ -2007,7 +2044,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             (cmdLineOpenDisplay && cmdLineWhereToPrint == INVALID)) {
          fprintf(stderr, TgLoadString(STID_PRINT_TO_PRINTER_INCOMPAT),
                "-one_file_page_page");
-         fprintf(stderr, "\n");
+         fprintf(stderr, "\n\n");
          Usage(progName);
       }
    }
@@ -2020,46 +2057,63 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
    /* All these strangeness with strings are related to */
    /*	Prolog's foreign function interface. */
 {
-   int i=0;
+   int i=0, len=0;
 
    *file_name = '\0';
    *cmdLineTmpFileModeStr = '\0';
    *cmdLineGenParser = '\0';
    *cmdLineFileToExec = '\0';
+   *cmdLineRMCastLibDir = *cmdLineRMCastLibPath = '\0';
+   *cmdLineCustomPatternDir = '\0';
    cmdLineMergeArgv = NULL;
    for (i=1; i < argc; i++) {
       if (strcmp(argv[i], "-display") == 0) {
          if (++i == argc) {
-            fprintf(stderr, "%s\n", TgLoadString(STID_MISSING_DISPLAY_NAME));
+            fprintf(stderr, "%s\n\n", TgLoadString(STID_MISSING_DISPLAY_NAME));
             Usage(progName);
          }
          if ((displayName=UtilStrDup(argv[i])) == NULL) FailAllocMessage();
+      } else if (strcmp(argv[i], "-tgrm2") == 0) {
+         cmdLineTgrm2 = TRUE;
+      } else if (strcmp(argv[i], "-rmcastlibdir") == 0) {
+         if (++i == argc) {
+            fprintf(stderr, "%s\n\n", TgLoadString(STID_MISSING_RMCASTDIR));
+            Usage(progName);
+         }
+         UtilStrCpyN(cmdLineRMCastLibDir, sizeof(cmdLineRMCastLibDir), argv[i]);
+      } else if (strcmp(argv[i], "-rmcastlibpath") == 0) {
+         if (++i == argc) {
+            fprintf(stderr, "%s\n\n", TgLoadString(STID_MISSING_RMCASTLIB));
+            Usage(progName);
+         }
+         UtilStrCpyN(cmdLineRMCastLibPath, sizeof(cmdLineRMCastLibPath),
+               argv[i]);
+      } else if (strcmp(argv[i], "-tgwb2") == 0) {
+         cmdLineWb2 = TRUE;
       } else if (strcmp(argv[i], "-bw") == 0) {
          cmdLineBW = TRUE;
-#if defined(_TANGRAM_II) || defined(_TGIF_WB2)
       } else if (strcmp(argv[i], "-parent") == 0) {
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-parent");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          if (strncmp(argv[i], "0x", 2) == 0) {
             if (sscanf(&(argv[i])[2], "%x", &cmdLineParentWindowID) != 1) {
                fprintf(stderr, TgLoadString(STID_FAIL_TO_PARSE_FOR_A_VAL),
                      argv[i]);
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
          } else {
             if (sscanf(argv[i], "%d", &cmdLineParentWindowID) != 1) {
                fprintf(stderr, TgLoadString(STID_FAIL_TO_PARSE_FOR_A_VAL),
                      argv[i]);
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
          }
-#endif /* defined(_TANGRAM_II) || defined(_TGIF_WB2) */
       } else if (strcmp(argv[i], "-reqcolor") == 0) {
          cmdLineRequestedColor = TRUE;
       } else if (strcmp(argv[i], "-rv") == 0) {
@@ -2078,24 +2132,24 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-exec");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          UtilStrCpyN(cmdLineFileToExec, sizeof(cmdLineFileToExec), argv[i]);
          UtilTrimBlanks(cmdLineFileToExec);
          if (*cmdLineFileToExec == '-' && cmdLineFileToExec[1] != '\0') {
-            fprintf(stderr, "%s\n", TgLoadString(STID_BAD_EXEC_FNAME_DASH));
+            fprintf(stderr, "%s\n\n", TgLoadString(STID_BAD_EXEC_FNAME_DASH));
             Usage(progName);
          }
       } else if (strcmp(argv[i], "-dbim") == 0) {
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-dbim");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          if (cmdLineHasInputMethod) {
-            fprintf(stderr, "%s\n", TgLoadString(STID_ONLYONE_DBIM_OR_SBIM));
+            fprintf(stderr, "%s\n\n", TgLoadString(STID_ONLYONE_DBIM_OR_SBIM));
             Usage(progName);
          }
          strcpy(cmdLineInputMethod, argv[i]);
@@ -2105,11 +2159,11 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-sbim");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          if (cmdLineHasInputMethod) {
-            fprintf(stderr, "%s\n", TgLoadString(STID_ONLYONE_DBIM_OR_SBIM));
+            fprintf(stderr, "%s\n\n", TgLoadString(STID_ONLYONE_DBIM_OR_SBIM));
             Usage(progName);
          }
          strcpy(cmdLineInputMethod, argv[i]);
@@ -2121,7 +2175,6 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
          cmdLineMerge = TRUE;
       } else if (strcmp(argv[i], "-a4") == 0) {
          cmdLineA4 = TRUE;
-#ifdef _TGIF_WB2
 #ifdef NOT_DEFINED
       } else if (strcmp(argv[i], "-listen") == 0) {
          cmdLineWhiteBoardListenOnly = TRUE;
@@ -2134,17 +2187,16 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-chatnick");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          cmdLineChatNickName = UtilStrDup(argv[i]);
          if (cmdLineChatNickName == NULL) FailAllocMessage();
-#endif /* _TGIF_WB2 */
       } else if (strcmp(argv[i], "-fg") == 0) {
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-fg");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          cmdLineForeground = (char*)malloc(80*sizeof(char));
@@ -2154,7 +2206,7 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-bg");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          cmdLineBackground = (char*)malloc(80*sizeof(char));
@@ -2164,7 +2216,7 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-bd");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          cmdLineBorder = (char*)malloc(80*sizeof(char));
@@ -2174,7 +2226,7 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-geometry");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          strcat(geometrySpec, argv[i]);
@@ -2204,16 +2256,38 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
                   "-pdfspd", tmp_str);
             fprintf(stderr, "\n");
          }
-      } else if (strcmp(*argv, "-pssetup") == 0) {
-         argc--; argv++;
-         if (argc <= 0) {
+      } else if (strcmp(argv[i], "-pssetup") == 0) {
+         i++;
+         if (i >= argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-pssetup");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          cmdLinePsSetup = TRUE;
-         SetPsSetup(*argv);
+         SetPsSetup(argv[i]);
+      } else if (strcmp(*argv, "-patterndir") == 0) {
+         argc--; argv++;
+         if (argc <= 0) {
+            fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
+                  "-patterndir");
+            fprintf(stderr, "\n\n");
+            Usage(progName);
+         }
+         UtilStrCpyN(cmdLineCustomPatternDir,
+               sizeof(cmdLineCustomPatternDir), *argv);
+         UtilTrimBlanks(cmdLineCustomPatternDir);
+         len = strlen(cmdLineCustomPatternDir);
+         if (len > 0 && cmdLineCustomPatternDir[len-1] == DIR_SEP) {
+            cmdLineCustomPatternDir[len-1] = '\0';
+         }
+         if (*cmdLineCustomPatternDir == '\0' ||
+               !UtilIsDirectory(cmdLineCustomPatternDir)) {
+            fprintf(stderr, TgLoadString(STID_PATTERNDIR_NOT_EXIST),
+                  cmdLineCustomPatternDir);
+            fprintf(stderr, "\n");
+            *cmdLineCustomPatternDir = '\0';
+         }
       } else if (argv[i][0] == '=') {
          strcpy(geometrySpec, argv[i]);
          geometrySpecified = TRUE;
