@@ -1,5 +1,5 @@
 /*
- * @(#)$Header: /mm/src/tgif/v4/RCS/nkf.c,v 4.5 2000/04/18 01:16:51 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/nkf.c,v 1.2 2009/01/12 05:29:50 william Exp $
  */
 
 /** Network Kanji Filter. (PDS Version)
@@ -221,8 +221,6 @@ static int fold();
 #define FOLD_MARGIN  10
 #define DEFAULT_FOLD 60
 /* converters */
-int             do_kconv ();
-
 static int      s_iconv ();
 
 static int      e_oconv ();
@@ -1359,7 +1357,7 @@ static int iso8859_f_save;
 static int
 mime_begin()
 {
-    int c1;
+    int c1=0;
     int i,j,k;
     unsigned char *p,*q;
 
