@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/page.c,v 1.3 2006/02/10 23:21:20 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/page.c,v 1.7 2009/08/13 01:14:20 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_PAGE_C_
@@ -1609,6 +1609,7 @@ void PrintPages()
    case PNG_FILE:
    case JPEG_FILE:
    case PPM_FILE:
+   case SVG_FILE:
       sprintf(gszMsgBox, TgLoadString(STID_CANT_PRINT_PAGES_IN_FORMAT),
             GetExportName(whereToPrint, EXPORT_THIS));
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
@@ -2054,7 +2055,7 @@ void PageEventHandler(input)
 void CleanUpPage()
 {
    DelAllPages();
-   if (leftend_bits) { }
+   if (leftend_bits == NULL) { }
 }
 
 static PageInfo stPageInfo;
