@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/remote.c,v 1.3 2006/02/10 23:21:20 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/remote.c,v 1.7 2009/09/28 22:39:48 william Exp $
  */
 
 #define _INCLUDE_FROM_REMOTE_C_
@@ -685,6 +685,12 @@ void UncompressTgifFile(url, ppsz_buf, pn_buf_sz, pn_html)
    } else {
       sprintf(cmd, uncompress_cmd, tmp_fname);
    }
+   if (!FindProgramInPath(cmd, NULL, FALSE)) {
+      free(cmd);
+      unlink(tmp_fname);
+      free(tmp_fname);
+      return;
+   }
    if ((pfp=(FILE*)popen(cmd, "r")) == NULL) {
       sprintf(gszMsgBox, TgLoadString(STID_CANNOT_EXECUTE_GIVEN_CMD), cmd);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
@@ -821,7 +827,8 @@ int LoadHttpIntoMem(url, host, port, path, ppsz_buf, ppsz_content_type,
                    * no matter what kind of file it is!
                    */
                   if (cmdLineDumpURL) {
-                     fwrite(*ppsz_buf, sizeof(char), (*pn_buf_sz), stdout);
+                     (void)fwrite(*ppsz_buf, sizeof(char), (*pn_buf_sz),
+                           stdout);
                   }
                }
             }
@@ -933,7 +940,7 @@ int LoadFtpIntoMem(url, host, port, path, ppsz_buf, ppsz_content_type,
              * no matter what kind of file it is!
              */
             if (cmdLineDumpURL) {
-               fwrite(*ppsz_buf, sizeof(char), (*pn_buf_sz), stdout);
+               (void)fwrite(*ppsz_buf, sizeof(char), (*pn_buf_sz), stdout);
             }
          }
          sprintf(gszMsgBox, TgLoadCachedString(CSTID_DATA_RECEIVED), "FTP");
@@ -1288,11 +1295,11 @@ void LaunchViewer(launch_remote_file, psz_viewer, psz_url,
 
 #ifdef _BACKGROUND_DONT_FORK
       strcat(cmd, " &");
-      system(cmd);
+      (void)system(cmd);
 #else /* ~_BACKGROUND_DONT_FORK */
       pid = fork();
       if (pid == 0) {
-         system(cmd);
+         (void)system(cmd);
          exit(0);
       }
 #endif /* _BACKGROUND_DONT_FORK */
