@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/scroll.c,v 1.3 2006/02/10 23:21:21 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/scroll.c,v 1.7 2009/08/13 01:14:20 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_SCROLL_C_
@@ -125,11 +125,22 @@ int TgPressButtonLoop(dpy, win, pbbox, psbci)
       initial_timeout = FALSE;
 
       if (status < 0) {
-         XUngrabPointer(dpy, CurrentTime);
-         sprintf(gszMsgBox, TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
-               "TgPressButtonLoop()");
-         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-         break;
+         if (errno == EINTR) {
+            /* interrupted by a system call, do it again */
+            if (TgAnyButtonDown(dpy, win)) {
+               need_to_scroll_once = FALSE;
+               if ((psbci->pf_scroll_btn_callback)(psbci->pv_userdata)) {
+                  XUngrabPointer(dpy, CurrentTime);
+                  return TRUE;
+               }
+            }
+         } else {
+            XUngrabPointer(dpy, CurrentTime);
+            sprintf(gszMsgBox, TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
+                  "TgPressButtonLoop()");
+            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+            break;
+         }
       } else if (status == 0 && TgAnyButtonDown(dpy, win)) {
          need_to_scroll_once = FALSE;
          if ((psbci->pf_scroll_btn_callback)(psbci->pv_userdata)) {
@@ -1517,7 +1528,7 @@ void GetOrigin(pn_x, pn_y, pn_zoom_scale, pn_zoomed_in)
 
 void CleanUpScrolls()
 {
-   if (scrl_up_bits) { }
+   if (scrl_up_bits == NULL) { }
 }
 
 void UpdScrollWinWH()
