@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/setup.c,v 1.20 2006/06/06 00:20:53 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/setup.c,v 1.58 2009/09/20 23:42:49 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_SETUP_C_
@@ -90,6 +90,7 @@
 #include "tangram2.e"
 #include "tdgtlist.e"
 #include "text.e"
+#include "tgcwheel.e"
 #include "tidget.e"
 #include "util.e"
 #include "version.e"
@@ -279,10 +280,12 @@ int	talkToSelfFiledes[2];
 
 Atom	compoundTextAtom=(Atom)0;
 Atom	textAtom=(Atom)0;
+Atom	utf8StringAtom=(Atom)0;
+Atom	tgifProtocolAtom=(Atom)0;
+Atom	tmpSelectionAtom=(Atom)0;
 
 static Atom	wmDeleteWindowAtom=(Atom)0;
 static Atom	targetsAtom=(Atom)0;
-static Atom	utf8StringAtom=(Atom)0;
 
 static int	baseScrollBarW=SCROLLBAR_W;
 
@@ -388,6 +391,7 @@ void InitWinSizes()
       pageDummyWindowW = PAGE_DUMMY_WINDOW_W + windowPadding;
    }
    pageWindowW = drawWinW+rulerW-hSBarW-pageDummyWindowW;
+   if (pageWindowW <= 0) pageWindowW = 1;
    colorWindowH = drawWinH+rulerW-DEF_VSBAR_H-colorDummyWindowH;
    vSBarH = drawWinH+rulerW+(brdrW<<1);
    titleWindowW = (noModeWindow ? 0 : modeWindowW+(brdrW<<1)) +
@@ -427,6 +431,7 @@ void InverseInitWinSizes()
          (noModeWindow ? 0 : modeWindowW+(brdrW<<1));
    modeWindowH = drawWinH+rulerW+(brdrW<<2);
    pageWindowW = titleWindowW-scrollBarW-hSBarW-pageDummyWindowW-(brdrW<<2);
+   if (pageWindowW <= 0) pageWindowW = 1;
    colorWindowH = drawWinH+rulerW-DEF_VSBAR_H-colorDummyWindowH;
    if (colorLayers) vSBarH = DEF_VSBAR_H;
    CalcMenubarWindowHeight();
@@ -475,8 +480,12 @@ void DoReconfigure()
    }
    if (!noChoiceWindow) {
       XMoveWindow(mainDisplay, choiceWindow, 0, cur_y);
-      XMoveResizeWindow(mainDisplay, msgWindow, choiceWindowW+(brdrW<<1), cur_y,
-            msgWindowW, msgWindowH);
+      if (msgWindowW > 0) {
+         XMoveResizeWindow(mainDisplay, msgWindow, choiceWindowW+(brdrW<<1),
+               cur_y, msgWindowW, msgWindowH);
+      } else {
+         XMoveWindow(mainDisplay, msgWindow, choiceWindowW+(brdrW<<1), cur_y);
+      }
       cur_y += msgWindowH + (brdrW<<1);
    }
    if (!noModeWindow) {
@@ -512,8 +521,12 @@ void DoReconfigure()
       draw_win_y_offset = cur_y;
    }
    cur_y += drawWinH + (brdrW<<1);
-   XMoveResizeWindow(mainDisplay, pageWindow, 0, cur_y, pageWindowW,
-         scrollBarW+(brdrW<<1));
+   if (pageWindowW > 0) {
+      XMoveResizeWindow(mainDisplay, pageWindow, 0, cur_y, pageWindowW,
+            scrollBarW+(brdrW<<1));
+   } else {
+      XMoveWindow(mainDisplay, pageWindow, 0, cur_y);
+   }
    XMoveWindow(mainDisplay, pageDummyWindow, pageWindowW, cur_y);
    XMoveWindow(mainDisplay, hSBarWindow,
          pageWindowW+pageDummyWindowW+(brdrW<<1), cur_y);
@@ -632,6 +645,7 @@ void Reconfigure(Forced)
       titleWindowW = initDrawWinW + rulerW + modeWindowW + scrollBarW +
             (brdrW*6);
    }
+   if (pageWindowW <= 0) pageWindowW = 1;
    menubarWindowW = titleWindowW;
    chatWindowW = titleWindowW;
    if (!noChatWindow) SetChatWindowGeom();
@@ -671,20 +685,57 @@ int TgIsCmdEvent(pXEv)
    return FALSE;
 }
 
+static
+void HandleSelectionClear(input)
+   XEvent *input;
+{
+   Window win=(Window)0;
+   Atom selection=(Atom)0;
+   Time time;
+
+   win = input->xselectionclear.window;
+   selection = input->xselectionclear.selection;
+   time = input->xselectionclear.time;
+
+#ifdef NOT_DEFINED
+   {
+      static int id=0;
+      char *atom_name=NULL;
+
+      printf("\n%1d: SelectionClear.\n", ++id);
+
+      atom_name = XGetAtomName(mainDisplay, selection);
+      printf("%1d: Selection name is '%s'.\n", id, atom_name);
+      XFree(atom_name);
+   }
+#endif /* NOT_DEFINED */
+   if (startSelectionOwnershipTimeValid) {
+      endSelectionOwnershipTime = time;
+      endSelectionOwnershipTimeValid = TRUE;
+   }
+   ClearSelection();
+}
+
 void HandleSelectionRequest(input)
    XEvent *input;
 {
    Window requestor=(Window)0;
    Atom property=(Atom)0, target=(Atom)0, selection=(Atom)0;
+   Time time=(Time)0;
    XEvent sent_ev;
 
    requestor = input->xselectionrequest.requestor;
    selection = input->xselectionrequest.selection;
    target = input->xselectionrequest.target;
    property = input->xselectionrequest.property;
+   time = input->xselectionrequest.time;
 
 #ifdef NOT_DEFINED
    {
+      /*
+       * After tgif does a Copy, tgif will get SelectionRequest at
+       *         a regular interval
+       */
       static int id=0;
       char *atom_name=NULL;
 
@@ -710,17 +761,32 @@ void HandleSelectionRequest(input)
    sent_ev.xselection.property = None;
 
    if (target == targetsAtom) {
-      Atom targets[2];
+      /*
+       * The selection manager is asking tgif what can you convert the
+       *         object copied into (i.e., what are the possobiel targets)?
+       */
+      Atom targets[5];
 
       targets[0] = XA_STRING;
       targets[1] = compoundTextAtom;
-      targets[2] = textAtom;
+      targets[2] = tgifProtocolAtom;
+      targets[3] = textAtom;
+      targets[4] = utf8StringAtom;
       XChangeProperty(mainDisplay, requestor, property, target, 32,
-            PropModeReplace, (unsigned char *)targets, 3);
+            PropModeReplace, (unsigned char *)targets, 5);
+      sent_ev.xselection.property = property;
    } else if (target == XA_STRING || target == textAtom) {
       XChangeProperty(mainDisplay, requestor, property, target, 8,
             PropModeReplace, (unsigned char *)(dsCutBuffer.s), dsCutBuffer.sz);
       sent_ev.xselection.property = property;
+   } else if (target == utf8StringAtom) {
+      XChangeProperty(mainDisplay, requestor, property, target, 8,
+            PropModeReplace, (unsigned char *)(dsCutBuffer.s), dsCutBuffer.sz);
+      sent_ev.xselection.property = property;
+   } else if (target == tgifProtocolAtom) {
+      XChangeProperty(mainDisplay, requestor, property, target, 8,
+            PropModeReplace, (unsigned char *)(dsCutBuffer.s), dsCutBuffer.sz);
+      sent_ev.xselection.property = property;
    } else if (target == compoundTextAtom) {
       XTextProperty text_property;
 
@@ -765,6 +831,9 @@ int mainWinEventHandler(input)
    } else if (input->type == SelectionRequest) {
       HandleSelectionRequest(input);
       return INVALID;
+   } else if (input->type == SelectionClear) {
+      HandleSelectionClear(input);
+      return INVALID;
    } else if (input->type == UnmapNotify) {
       Iconify();
    } else if (input->type == MapNotify) {
@@ -832,6 +901,19 @@ int mainWinEventHandler(input)
                   key_sym = (KeySym)(key_ev.keycode);
                   TranslateKeys(buf, &key_sym);
                }
+               /*
+                * This is probabl not needed.
+                *
+                * ClearCopyUTF8Info(); 
+                * if (textHighlight) {
+                *    if (!CanCopyHighLightedTextAsUTF8Strings(NULL)) {
+                *       ClearCopyUTF8Info();
+                *    } else if (gstCopyUTF8Info.double_byte_valid &&
+                *          !gstCopyUTF8Info.single_byte_valid) {
+                *       CalcSingleByteInfoForCopyUTF8(NULL); 
+                *    }
+                * }
+                */
                HandleTextFromBuffer(&key_ev, buf, key_sym, has_ch);
             }
          } else {
@@ -891,11 +973,10 @@ void InitPaperSize()
    psPageHeightInInch[1] = ((float)onePageWidth)/((float)ONE_INCH);
 }
 
-static Window	savedMainWindow=None, savedDrawWindow=None;
 static Pixmap	wmIconPixmap=None, wmIconPixmapMask=None;
 
 static
-int GetIconPixmap(pPixmap, pPixmapMask)
+int GetWMIconPixmap(pPixmap, pPixmapMask)
    Pixmap *pPixmap, *pPixmapMask;
 {
    Pixmap pixmap=None, bitmap=None;
@@ -903,7 +984,8 @@ int GetIconPixmap(pPixmap, pPixmapMask)
    int rc=0, image_w=0, image_h=0, w=0, h=0, ncolors=0, chars_per_pixel=0;
    int first_pixel_is_bg=FALSE, *pixels=NULL;
    char fname[MAXPATHLENGTH], *color_char=NULL, **color_str=NULL;
-   char *c_ptr=XGetDefault(mainDisplay,TOOL_NAME, "IconPixmap"), *xpm_data=NULL;
+   char *xpm_data=NULL;
+   char *c_ptr=XGetDefault(mainDisplay,TOOL_NAME, "WMIconPixmap");
 
    *pPixmap = *pPixmapMask = None;
    if (c_ptr == NULL) return FALSE;
@@ -1068,7 +1150,7 @@ void Setup()
    }
 #endif /* _TGIF_DBG */
    if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "Synchronize")) != NULL) {
-      if (UtilStrICmp(c_ptr, "on") == 0) {
+      if (UtilStrICmp(c_ptr, "on") == 0 || UtilStrICmp(c_ptr, "true") == 0) {
          XSynchronize(mainDisplay, True);
       }
    }
@@ -1132,6 +1214,8 @@ void Setup()
          whereToPrint = PPM_FILE;
       } else if (UtilStrICmp(c_ptr, "NetList") == 0) {
          whereToPrint = NETLIST_FILE;
+      } else if (UtilStrICmp(c_ptr, "SVG") == 0) {
+         whereToPrint = SVG_FILE;
       } else {
          fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
                TOOL_NAME, "WhereToPrint", c_ptr, "EPS");
@@ -1210,10 +1294,11 @@ void Setup()
          splineTol = 9;
       }
    }
-   splineRubberband = TRUE;
    if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"SplineRubberband")) != NULL) {
       if (UtilStrICmp(c_ptr, "false") == 0) {
-         splineRubberband = FALSE;
+         sprintf(gszMsgBox, TgLoadString(STID_NAMED_XDEF_IS_OBSOLETE),
+               TOOL_NAME, "SplineRubberband");
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       }
    }
    saveTmpOnReturn = TRUE;
@@ -1273,7 +1358,7 @@ void Setup()
          btn1Warp = TRUE;
       } else if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,
             "QuickWarp")) != NULL) {
-         if (UtilStrICmp("on", c_ptr) == 0) {
+         if (UtilStrICmp("on", c_ptr) == 0 || UtilStrICmp("true", c_ptr) == 0) {
             btn1Warp = TRUE;
          }
       } else if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,
@@ -1386,19 +1471,19 @@ void Setup()
          }
       }
    }
-#ifdef _TGIF_WB2
-   if (cmdLineNoChatWindow) {
-      noChatWindow = TRUE;
-   } else {
-      noChatWindow = FALSE;
-      if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"NoChatWindow")) != NULL) {
-         if (UtilStrICmp("true", c_ptr) == 0) {
-            noChatWindow = TRUE;
+   if (gstWBInfo.do_whiteboard) {
+      if (cmdLineNoChatWindow) {
+         noChatWindow = TRUE;
+      } else {
+         noChatWindow = FALSE;
+         if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"NoChatWindow"))
+               != NULL) {
+            if (UtilStrICmp("true", c_ptr) == 0) {
+               noChatWindow = TRUE;
+            }
          }
       }
    }
-#endif /* _TGIF_WB2 */
-
    scrollBarW = baseScrollBarW = SCROLLBAR_W;
    if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "ScrollBarWidth")) != NULL) {
       baseScrollBarW = atoi(c_ptr);
@@ -1423,6 +1508,9 @@ void Setup()
    wmDeleteWindowAtom = XInternAtom(mainDisplay, "WM_DELETE_WINDOW", False);
    targetsAtom = XInternAtom(mainDisplay, "TARGETS", False);
    utf8StringAtom = XInternAtom(mainDisplay, "UTF8_STRING", False);
+   tgifProtocolAtom = XInternAtom(mainDisplay, TGIF_PROTOCOL_ATOM, False);
+   tmpSelectionAtom = XInternAtom(mainDisplay, "TGIF_TMP_SELECTION_ATOM",
+         False);
    compoundTextAtom = XInternAtom(mainDisplay, "COMPOUND_TEXT", False);
    textAtom = XInternAtom(mainDisplay, "TEXT", False);
    executeCmdByIDAtom = XInternAtom(mainDisplay, "TGIF_EXECUTE_CMD_BY_ID",
@@ -1640,14 +1728,10 @@ void Setup()
          mainVisual->class==StaticGray) ? FALSE : TRUE;
    fullTrueColorMode = FALSE;
    if (colorDisplay && mainVisual->class == TrueColor) {
-#ifdef NOT_DEFINED
-      if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "UseCompatColorScheme")) !=
+      if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"EnableTrueColorImages")) !=
             NULL && UtilStrICmp(c_ptr, "true") == 0) {
-         fullTrueColorMode = FALSE;
-      } else {
          fullTrueColorMode = TRUE;
       }
-#endif /* NOT_DEFINED */
    }
    noAntiAlias = TRUE;
    if (fullTrueColorMode) {
@@ -1805,7 +1889,7 @@ void Setup()
       }
       drawWinW = initDrawWinW = mainWinW-(brdrW<<1);
       drawWinH = initDrawWinH = mainWinH-(brdrW<<1);
-      if (msgWindowW <= 0) msgWindowW = 2;
+      if (msgWindowW <= 0) msgWindowW = 1;
    } else {
       sizehints.min_width =
             (noModeWindow ? 0 : modeWindowW+(brdrW<<1))+
@@ -1838,16 +1922,20 @@ void Setup()
    } else if (!y_in_cmdline && y_neg_in_def) {
       sizehints.y = def_y_neg+DisplayHeight(mainDisplay,mainScreen)-mainWinH-1;
    }
-#if defined(_TANGRAM_II) || defined(_TGIF_WB2)
-   if ( cmdLineParentWindowID > 0 ) {
-      rootWindow = cmdLineParentWindowID;
-   }
-#endif /* defined(_TANGRAM_II) || defined(_TGIF_WB2) */
    if ((mainWindow=XCreateSimpleWindow(mainDisplay, rootWindow,
          sizehints.x, sizehints.y, sizehints.width, sizehints.height,
          brdrW, myBorderPixel, myBgPixel)) == 0) {
       FailToCreateWindowMessage("Setup()", NULL, TRUE);
    }
+   if (gstWBInfo.do_whiteboard) {
+      if ( cmdLineParentWindowID > 0 ) {
+         XReparentWindow(mainDisplay, mainWindow, cmdLineParentWindowID, 0, 0);
+      }
+   } else {
+      if ( cmdLineTgrm2 && cmdLineParentWindowID > 0 ) {
+         XReparentWindow(mainDisplay, mainWindow, cmdLineParentWindowID, 0, 0);
+      }
+   }
    if (newColormapUsed) {
       XSetWindowColormap(mainDisplay, mainWindow, mainColormap);
    }
@@ -1989,31 +2077,20 @@ void Setup()
       }
    }
    InitTidget();
+   InitColorWheel();
+
+   if (!CreateChatSubWindows()) {
+      FailToCreateWindowMessage("Setup()", NULL, TRUE);
+   }
    if (!noChatWindow) {
-      if (!CreateChatSubWindows()) {
-         FailToCreateWindowMessage("Setup()", NULL, TRUE);
-      }
       cur_y += chatWindowH + (brdrW<<1);
    }
    if (!noStatusWindow) {
       cur_y += statusWindowH + (brdrW<<1);
    }
    if (canvasWindowOnly) {
-      savedMainWindow = mainWindow;
-      savedDrawWindow = drawWindow;
-
-      if ((mainWindow=XCreateSimpleWindow(mainDisplay, rootWindow,
-            sizehints.x, sizehints.y, mainWinW, mainWinH,
-            brdrW, myBorderPixel, bg_pixel)) == 0) {
-         FailToCreateWindowMessage("Setup()", NULL, TRUE);
-      }
-      if (newColormapUsed) {
-         XSetWindowColormap(mainDisplay, mainWindow, mainColormap);
-      }
-      if ((drawWindow=XCreateSimpleWindow(mainDisplay, mainWindow,
-            0, 0, drawWinW, drawWinH, brdrW, myBorderPixel, myBgPixel)) == 0) {
-         FailToCreateWindowMessage("Setup()", NULL, TRUE);
-      }
+      XMoveResizeWindow(mainDisplay, drawWindow, 0, 0, drawWinW, drawWinH);
+      XResizeWindow(mainDisplay, mainWindow, mainWinW, mainWinH);
    }
 
 #ifdef NOTR4MODE
@@ -2052,10 +2129,9 @@ void Setup()
    InitNKF();
 #endif /* ~_NO_NKF */
 
-#ifdef _TANGRAM_II
-   InitTangram2();
-#endif /* _TANGRAM_II */
-
+   if (cmdLineTgrm2) {
+      InitTangram2();
+   }
    InitShortCut();
    InitHtml();
    InitRecentFiles();
@@ -2087,8 +2163,9 @@ void Setup()
             PropertyChangeMask);
    } else {
       XMapWindow(mainDisplay, titleWindow);
-      XSelectInput(mainDisplay, titleWindow,
-            ExposureMask | EnterWindowMask | ButtonPressMask);
+      XSelectInput(mainDisplay, titleWindow, ExposureMask |
+            EnterWindowMask | LeaveWindowMask | ButtonPressMask |
+            PointerMotionMask);
 
       if (menubarWindow != None) XMapWindow(mainDisplay, menubarWindow);
       XSelectInput(mainDisplay, menubarWindow, ExposureMask |
@@ -2152,12 +2229,12 @@ void Setup()
 
       if (!noChatWindow) {
          XMapWindow(mainDisplay, chatWindow);
-         XSelectInput(mainDisplay, chatWindow,
-               KeyPressMask | ButtonPressMask | ExposureMask |
-               StructureNotifyMask | EnterWindowMask | LeaveWindowMask);
-
-         MapChatSubWindows();
       }
+      MapChatSubWindows();
+      XSelectInput(mainDisplay, chatWindow,
+            KeyPressMask | ButtonPressMask | ExposureMask |
+            StructureNotifyMask | EnterWindowMask | LeaveWindowMask);
+
       if (!noStatusWindow) {
          XMapWindow(mainDisplay, userRedrawWindow);
          XSelectInput(mainDisplay, userRedrawWindow, ButtonPressMask |
@@ -2186,8 +2263,9 @@ void Setup()
             PropertyChangeMask);
       XMapWindow(mainDisplay, drawWindow); 
    } else {
-      XSelectInput(mainDisplay, titleWindow,
-            ExposureMask | EnterWindowMask | ButtonPressMask);
+      XSelectInput(mainDisplay, titleWindow, ExposureMask |
+            EnterWindowMask | LeaveWindowMask | ButtonPressMask |
+            PointerMotionMask);
       XMapWindow(mainDisplay, titleWindow);
 
       XSelectInput(mainDisplay, menubarWindow, ExposureMask |
@@ -2252,13 +2330,14 @@ void Setup()
       XSelectInput(mainDisplay, dummyWindow2, ExposureMask | EnterWindowMask);
       XMapWindow(mainDisplay, dummyWindow2);
 
+      XSelectInput(mainDisplay, chatWindow,
+            KeyPressMask | ButtonPressMask | ExposureMask |
+            StructureNotifyMask | EnterWindowMask | LeaveWindowMask);
       if (!noChatWindow) {
          XMapWindow(mainDisplay, chatWindow);
-         MapChatSubWindows();
-         XSelectInput(mainDisplay, chatWindow,
-               KeyPressMask | ButtonPressMask | ExposureMask |
-               StructureNotifyMask | EnterWindowMask | LeaveWindowMask);
       }
+      MapChatSubWindows();
+
       if (!noStatusWindow) {
          XSelectInput(mainDisplay, userRedrawWindow, ButtonPressMask |
                ExposureMask | EnterWindowMask);
@@ -2327,7 +2406,7 @@ void Setup()
             NULL) || UtilStrICmp(c_ptr, "false") != 0) {
          wmhints.icon_mask = None;
          wmhints.initial_state = NormalState;
-         if (!GetIconPixmap(&wmIconPixmap, &wmIconPixmapMask)) {
+         if (!GetWMIconPixmap(&wmIconPixmap, &wmIconPixmapMask)) {
             wmhints.icon_pixmap = wmIconPixmap =
                   XCreateBitmapFromData(mainDisplay, mainWindow,
                   (char*)tgificon_bits, tgificon_width, tgificon_height);
@@ -2369,13 +2448,12 @@ void CleanUpResiduals()
       XFreePixmap(mainDisplay, wmIconPixmapMask);
       wmIconPixmapMask = None;
    }
-   if (savedMainWindow != None) XDestroyWindow(mainDisplay, savedMainWindow);
    CleanUpPaperSize();
    if (displayName != NULL) {
       UtilFree(displayName);
       displayName = NULL;
    }
-   if (btn1_bits) { }
+   if (btn1_bits == NULL) { }
 }
 
 int TieLooseEnds()
