@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/util.c,v 1.5 2006/06/09 23:00:00 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/util.c,v 1.9 2009/01/09 22:25:32 william Exp $
  */
 
 #define _INCLUDE_FROM_UTIL_C_
@@ -552,12 +552,13 @@ int UtilPathExists(pszPath)
    return (stat(pszPath, &stat_buf) == 0);
 }
 
-int UtilPathIsLink(char *pszPath)
-    /*
-     *      * if pszPath is not valid, return FALSE
-     *           * if pszPath is valid but not a link, return FALSE
-     *                * if pszPath is valid and is a link, return TRUE
-     *                     */
+int UtilPathIsLink(pszPath)
+   char *pszPath;
+   /*
+    * if pszPath is not valid, return FALSE
+    * if pszPath is valid but not a link, return FALSE
+    * if pszPath is valid and is a link, return TRUE
+    */
 {
 #ifndef _NO_LSTAT
     struct stat stat_buf;
@@ -579,7 +580,8 @@ int UtilIsDirectory(pszPath)
       (stat_buf.st_mode & S_IFDIR) == S_IFDIR);
 }
 
-int UtilCreateDirHier(char *pszDir)
+int UtilCreateDirHier(pszDir)
+   char *pszDir;
 {
     char *psz=NULL;
     int ok=TRUE;
@@ -615,49 +617,6 @@ int UtilCreateDirHier(char *pszDir)
     return ok;
 }
 
-char *UtilMkTempFile(char *buf, int buf_sz, char *pszDir, char *pszPrefix)
-{
-    int fd=(-2), fd_used=FALSE;
-
-#ifdef _TGIF_DBG /* debug, do not translate */
-    if (buf_sz <= sizeof(char*)) {
-       fprintf(stderr, "buf_sz <= sizeof(char*) in UtilMkTempFile()\n");
-    }
-#endif /* _TGIF_DBG */
-
-#ifndef _DONT_USE_MKTEMP
-    sprintf(buf, "%s%sXXXXXX", pszDir, pszPrefix);
-    mktemp(buf);
-#ifdef O_EXCL
-    fd_used = TRUE;
-    fd = open(buf, O_CREAT|O_EXCL|O_WRONLY);
-#endif /* O_EXCL */
-#else /* _DONT_USE_MKTEMP */
-    sprintf(buf, "%s%sXXXXXX", pszDir, pszPrefix);
-    fd_used = TRUE;
-    fd = mkstemp(buf);
-#endif /* ~_DONT_USE_MKTEMP */
-
-    if (fd_used) {
-        if (fd == (-1)) {
-            /* barf loudly that someone is trying to hack the box */
-            char gszMsgBox[MAXSTRING<<1];
-    
-            sprintf(gszMsgBox, TgLoadString(STID_RACE_IN_CREATE_TMP_FILE), buf);
-            if (PRTGIF) {
-               fprintf(stderr, "%s\n", gszMsgBox);
-            } else {
-               MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-            }
-        } else {
-            close(fd);
-        }
-    }
-    unlink(buf);
-    
-    return buf;
-}
-
 #ifndef _NO_GETTIMEOFDAY
 static
 long ms_time(tv)
@@ -667,7 +626,8 @@ long ms_time(tv)
 }
 #endif /* ~_NO_GETTIMEOFDAY */
 
-long UtilGetMilliSecTime(long *pl_sec, long *pl_msec)
+long UtilGetMilliSecTime(pl_sec, pl_msec)
+   long *pl_sec, *pl_msec;
 {
 #ifdef _NO_GETTIMEOFDAY
    struct timeb start;
